"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
function waitForDOMInactivity(options) {
    if (options === void 0) { options = {
        minInactivityTimeMs: 1000,
        maxInactivityTimeMs: 5000,
        intervalMs: 100,
    }; }
    Cypress.log({
        name: "waitForDOMInactivity",
        message: "Wait for ".concat(options.minInactivityTimeMs, " of DOM inactivity."),
        consoleProps: function () { return options; },
    });
    var lastMutation = Date.now();
    var observer = new MutationObserver(observerCallback);
    observer.observe(document.body, {
        attributes: true,
        childList: true,
        subtree: true,
    });
    var waitTimeStart = Date.now();
    function observerCallback() {
        lastMutation = Date.now();
    }
    return cy.then(function () {
        return new Cypress.Promise(function (resolve, reject) {
            function checkInactivityTime() {
                var totalWaitingTime = Date.now() - waitTimeStart;
                var timeSinceLastMutation = Date.now() - lastMutation;
                if (timeSinceLastMutation > options.minInactivityTimeMs) {
                    console.log("DOM activity is stable.");
                    // observer.disconnect();
                    Cypress.log({
                        name: "waitForDOMInactivity",
                        message: "DOM took ".concat(totalWaitingTime, " to stop mutating"),
                        consoleProps: function () { return options; },
                    });
                    resolve();
                }
                else if (totalWaitingTime > options.maxInactivityTimeMs) {
                    reject(new Error("DOM did not stop changing after ".concat(options.maxInactivityTimeMs, "ms")));
                }
                else {
                    setTimeout(checkInactivityTime, options.intervalMs);
                }
            }
            setTimeout(checkInactivityTime, options.intervalMs);
        });
    });
}
exports.default = waitForDOMInactivity;
Cypress.Commands.add('waitForDOMInactivity', function (options) {
    if (options === void 0) { options = {
        minInactivityTimeMs: 1000,
        maxInactivityTimeMs: 5000,
        intervalMs: 100,
    }; }
    return waitForDOMInactivity(options);
});
